-- func_to_bcd.vcd V 1.2 9/29/05
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


package func_to_bcd is
    function to_bcd (binary: std_logic_vector) return std_logic_vector; 
end func_to_bcd;    

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
package body func_to_bcd is
-----------------------------------------------------------------------
--Function: to_bcd - Converts 16 bit input to BCD. If input > 9999 then
--          returns x"9999".
--   Input: binary - 16 bit unsigned value.
-- Returns: BCD unsigned.
--  Ussage: BCD = to_bcd(binary);
-----------------------------------------------------------------------
    function to_bcd(binary: std_logic_vector) return STD_LOGIC_VECTOR is
        variable rsr : std_logic_vector(15 downto 0); -- Shift reg for BCD result
        variable bsr : std_logic_vector(15 downto 0); -- Shift reg for binary input
        variable correction : std_logic_vector(15 downto 0); -- Add to do BCD correction
    begin
        if CONV_INTEGER(binary) > 9999 then
            rsr := x"9999";
        else
            rsr := (others => '0'); 
            bsr := binary;   
            for i in 1 to 16 loop
                rsr := rsr(14 downto 0) & bsr(15);
                bsr := bsr(14 downto 0) & '0';
                if i < 16 then
                    for j in 0 to 3 loop
                        if rsr(4*j+3 downto 4*j) >= "0101" then
                            correction(4*j+3 downto 4*j) := "0011";
                        else
                            correction(4*j+3 downto 4*j) := "0000";
                        end if;
                    end loop;
                    rsr := rsr + correction;
                end if;                        
            end loop;
        end if;
        return rsr;    
    end to_bcd; 
 end func_to_bcd;